@@ -64,15 +64,21 @@ class Framer(tasking.Tasker):
             .frameNames = frame name registry , name space of frame names
             .frameCounter = frame name registry counter
 
-            .moots = list of moot framers to be cloned
+            .moots = odict of moot framers to be cloned keyed by clone tag
             .inode = prefix string for inode ioinit of do verb objects in framer
+            .tag = main framer local unique clone tag when cloned or aux name if not
+            .insularCount = number of insular clones used to create unique clone tag
+            .auxes = odict of cloned auxes keyed by tag name of clone
     """
     #Counter = 0
     #Names = {}
 
-    def __init__(self, **kw):
-        """Initialize instance.
+    def __init__(self, tag='', **kw):
+        """
+        Initialize instance.
 
+        Parameters:
+            tag = unique to framer clone tag name if clone otherwise same as .name if empty
 
         """
         super(Framer,self).__init__(**kw) #status = STOPPED  make runner advance so can send cmd
@@ -109,12 +115,40 @@ class Framer(tasking.Tasker):
         self.frameNames = odict() #frame name registry for framer. name space of frame names
         self.frameCounter = 0 #frame name registry counter for framer
 
-        self.moots = []  # moot framers to be cloned
-        self.clones = odict()  # cloned framers keyed by long name of clone
+        self.moots = odict()  # moot framers to be cloned keyed by clone tag
         self.inode = ''  # framer inode prefix
 
-    def clone(self, name, period=0.0, schedule=AUX):
-        """ Return clone of self named name
+        self.tag = tag if tag else self.name  # main framer local unique clone tag when cloned or .name if not
+        self.auxes = odict()  # aux framers keyed by clone tag if clone or aux name if not
+
+    @property
+    def mains(self):
+        """
+        Property that returns tuple of framer hierarchy of current framer
+        from current framer to an including first non clone framer
+        (framername,clonetag,clonetag,clonetag)
+        """
+        framer = self
+        mains = [framer]
+        while not framer.original:  # stop at first original framer (not clone)
+            framer = framer.main.framer  # assumes .main is resolved
+            mains.append(framer)
+
+        return tuple(reversed(mains))
+
+    @property
+    def surname(self):
+        """
+        Property that returns underscore separated string representing namespaced
+        clone framer hierarchy
+        For non cloned framers the .surname should be the same as .name
+        Uses .mains property so assumes that .main is resolved if any
+        """
+        return "_".join(framer.name if framer.original else framer.tag for framer in self.mains)
+
+    def clone(self, name, tag='', period=0.0, schedule=AUX,):
+        """
+        Return clone of self named name with tag
 
         """
         self.store.house.assignRegistries() # ensure Framer.names is houses registry
@@ -130,7 +164,8 @@ class Framer(tasking.Tasker):
         clone = Framer(name=name,
                        store=self.store,
                        period=period,
-                       schedule=schedule)
+                       schedule=schedule,
+                       tag=tag)
         console.terse("         Cloning contents of Framer original '{0}' to clone '{1}'\n"
                         "".format(self.name, clone.name))
         clone.schedule = schedule
@@ -148,6 +183,7 @@ class Framer(tasking.Tasker):
         """
         Recursively Prune (destroy) all insular auxiliary clones in all frames
         Force exit if not done
+        Called by Razer Actor when razing insular auxes from frame
         """
         if not self.done:
             console.profuse("Force exiting '{0}'\n".format(self.name))
@@ -158,25 +194,47 @@ class Framer(tasking.Tasker):
             for aux in prunables:
                 aux.prune()
                 frame.auxes.remove(aux)
+                if aux.tag in self.auxes:
+                    del self.auxes[aux.tag]
 
         if self.name in Framer.Names and Framer.Names[self.name] == self:
             del Framer.Names[self.name]
 
-    def resolve(self):
-        """Convert all the name strings for links to references to instance
-           by that name
+    def presolve(self):
         """
-        console.terse("     Resolving Framer {0}\n".format(self.name))
+        Convert namestrings or data of moots into clones
+        """
+        console.terse("     Presolving Framer {0}\n".format(self.name))
         self.resolveMoots()
 
-        self.assignFrameRegistry() #needed by act links below
+        self.assignFrameRegistry()  # needed so Frame.Names valid
 
-        #Resolve first frame link
+        # Resolve first frame link
         if self.first:
             self.first = resolveFrame(self.first, who=self.name, desc='first')
         else:
             raise excepting.ResolveError("No first frame link", self.name, self.first)
 
+
+        console.terse("       Presolving frames for framer {0}\n".format(self.name))
+        for frame in Frame.Names.values():  # all frames in this framer's name space
+            frame.presolve()
+
+        self.presolved = True
+
+    def resolve(self):
+        """
+        Convert all the name strings for links to references to instance
+        by that name
+        """
+        if not self.presolved:
+            raise excepting.ResolveError("Not presolved", self.name, self.main)
+
+        console.terse("     Resolving Framer {0}\n".format(self.name))
+
+        self.assignFrameRegistry() #needed by act links below
+
+        console.terse("       Resolving frames for framer {0}\n".format(self.name))
         for frame in Frame.Names.values(): #all frames in this framer's name space
             frame.resolve()
 
@@ -187,7 +245,7 @@ class Framer(tasking.Tasker):
     def resolveMoots(self):
         """
         Resolves .moots by cloning as appropriate.
-        .moots is list of dicts of form
+        .moots is odict keyed by clone tag with value that are dicts of form
         {
             original: framername,
             clone: string,
@@ -195,6 +253,7 @@ class Framer(tasking.Tasker):
             human: string,
             count: number,
             inode: pathstring,
+            insular: boolean
         }
 
         creates clone and adds to taskables if named clone
@@ -205,16 +264,25 @@ class Framer(tasking.Tasker):
         """
         #self.store.house.assignRegistries() # ensure Framer.names is houses registry
         console.terse("       Resolving original moots for named clones ...\n")
-        for data in self.moots:
+        for tag, data in self.moots.items():
             original = data['original']  # original name
-            clone = data['clone']  # clone name
+            name = data['clone']  # clone name tag should match tag key
             schedule = data['schedule']
             human = data['human']
             count = data['count']
             inode = data['inode']
-            console.terse("         Cloning original '{0}' as named clone '{1}'\n"
-                            "".format(original, clone))
-            if clone == 'mine':  # invalid name for named clone
+            insular = data['insular']
+            console.terse("         Cloning original '{0}' as {1} clone '{2}'\n"
+                            "".format(original,
+                                      "insular" if insular else "named",
+                                      name))
+            if name != tag:
+                raise excepting.ResolveError("Mismatch clone tag",
+                                                             name=name,
+                                                             value=tag,
+                                                             human=human,
+                                                             count=count )
+            if name == 'mine':  # invalid name for named clone
                 raise excepting.ResolveError("Invalid named clone name of 'mine'",
                                              name=original,
                                              value=self.name,
@@ -227,20 +295,56 @@ class Framer(tasking.Tasker):
                                      contexts=[MOOT],
                                      human=human,
                                      count=count)
-            clone = original.clone(name=clone, schedule=schedule)
+
+            if tag in self.auxes:  # tag must be unique to framer
+                raise excepting.ResolveError("Clone tag already in use",
+                                             name=self.name,
+                                             value=tag,
+                                             human=human,
+                                             count=count )
+            name = "_".join((self.surname, tag))  # replace name with full name
+            clone = original.clone(name=name, tag=tag, schedule=schedule)
+            self.auxes[tag] = clone
 
             # inode is new (aux verb clone via)  clone.inode is old (framer moot via)
-            if inode:  # new != "" so resultant is new
+            if inode != "mine":  # new != "mine" so resultant is new
                 clone.inode = inode  # replace old with new
 
-            clone.original = False  # main frame will be fixed
-            self.store.house.resolvables.append(clone)
+            clone.original = False  # fixed main frame value, unique main frame
+            clone.insular = insular  # local to this framer can not be referenced
+            self.store.house.presolvables.append(clone)  # so can resolve
             self.store.house.taskers.append(clone)
             self.store.house.framers.append(clone)
             if schedule == AUX:
                 self.store.house.auxes.append(clone)
 
-        self.moots = []
+        self.moots.clear()
+
+    def newMootTag(self, base=None, count=0):
+        """
+        Returns new unique among .moots insular tag by incrementing count
+        """
+        if not base:
+            base = self.tag
+        count += 1
+        tag = "{0}{1}".format(base, count)
+        while tag in self.moots:
+            count += 1
+            tag = "{0}{1}".format(base, count)
+        return tag
+
+    def newAuxTag(self, base=None, count=0):
+        """
+        Returns new unique among .auxes insular tag by using .insularCount
+        """
+        if not base:
+            base = self.tag
+        count += 1
+        tag = "{0}{1}".format(base, count)
+        while tag in self.auxes:
+            count += 1
+            tag = "{0}{1}".format(base, count)
+        return tag
 
     @staticmethod
     def nameUid(prefix='clone', size=8):
@@ -860,21 +964,33 @@ class Frame(registering.StoriedRegistrar):
 
         return clone
 
-    def resolve(self):
-        """Resolve links where links are instance name strings assigned during building
+    def presolve(self):
+        """
+        Resolve auxiliary links where links are instance name strings assigned during building
            need to be converted to object references using instance name registry
 
         """
-        console.concise("       Resolving Frame {0}\n".format(self.name))
+        console.concise("        Presolving Frame {0}\n".format(self.name))
 
         self.resolveFramerLink()
+        self.resolveAuxLinks()
+
+
+    def resolve(self):
+        """
+        Resolve links where links are instance name strings assigned during building
+           need to be converted to object references using instance name registry
+
+        """
+        console.concise("        Resolving Frame {0}\n".format(self.name))
+
+        #self.resolveFramerLink()
+        #self.resolveAuxLinks()
 
         self.resolveNextLink()
         self.resolveOverLinks()
         self.resolveUnderLinks()
 
-        self.resolveAuxLinks()
-
         for act in self.beacts:
             act.resolve()
 
@@ -903,6 +1019,69 @@ class Frame(registering.StoriedRegistrar):
                                                 who=self.name,
                                                 desc="frame's",)
 
+    def resolveAuxLinks(self):
+        """
+        Resolve aux links
+
+        If aux.original
+           aux.main for each aux is not assigned here but is assigned when
+          frame.enter before aux.enterAll() so can reuse aux in other frames.
+        Otherwise
+           assign aux.main to self
+        """
+        for i, aux in enumerate(self.auxes):
+            cloned = False
+            if isinstance(aux, Mapping):  # Indicates a clone  (see builder.buildAux)
+                tag = aux.get('tag')
+                if not tag:
+                    msg = "Empty aux clone tag link '{0}'".format(aux)
+                    raise excepting.ResolveError(msg,name=self.name,value=aux)
+                aux = tag
+                cloned = True
+
+            if not isinstance(aux, Framer): # not instance so aux is a name or tag
+                if cloned:  # should already be entered in framer.auxes
+                    self.auxes[i] = aux = resolveAuxOfFramer(aux,
+                                                                 self.framer,
+                                                                 who=self.name,
+                                                                 desc='aux',
+                                                                 contexts=[AUX],
+                                                                 human=self.human,
+                                                                 count=None)
+                    if aux.original:  # whoops
+                        msg = ("Aux clone tag already in use by original aux '{0}' in"
+                               " '{1}'".format(aux.name, self.framer.name))
+                        raise excepting.ResolveError(msg, name=aux.name, value=self.name)
+
+                    # aux is clone, clones get fixed main never released
+                    if aux.main: # raise exception if aux.main is not None
+                        msg = "Aux already assigned to main '{0}'".format(aux.main.name)
+                        raise excepting.ResolveError(msg,
+                                                     name=aux.name,
+                                                     value=self.name)
+                    aux.main = self
+
+                else:  # not clone may not be entered
+                    self.auxes[i] = aux = resolveFramer(aux,
+                                                        who=self.name,
+                                                        desc='aux',
+                                                        contexts=[AUX],
+                                                        human=self.human,
+                                                        count=None)
+                    if not aux.original:  # must be original
+                        msg = ("Unexpected non-original aux name '{0}' in"
+                               " '{1}'".format(aux.name, self.framer.name))
+                        raise excepting.ResolveError(msg, name=aux.name, value=self.name)
+
+                    if aux.name not in self.framer.auxes:  # first time
+                        self.framer.auxes[aux.name] = aux  # add for the first time
+                    else:
+                        if self.framer.auxes[aux.name] is not aux:  # must be same aux
+                            msg = ("Aux name '{0}' already in use in"
+                                   " '{1}'".format(aux.name, self.framer.name))
+                            raise excepting.ResolveError(msg, name=aux.name, value=self.name)
+
+
     def resolveNextLink(self):
         """Resolve next link
 
@@ -954,76 +1133,6 @@ class Frame(registering.StoriedRegistrar):
         if len(set(self.unders)) != len(self.unders): # duplicates
             raise excepting.ResolveError("Duplicate under", name=self.name, value=self.unders)
 
-    def resolveAuxLinks(self):
-        """ Resolve aux links
-
-            If aux.original
-               aux.main for each aux is not assigned here but is assigned when
-              frame.enter before aux.enterAll() so can reuse aux in other frames.
-            Otherwise
-               assign aux.main to self
-        """
-        for i, aux in enumerate(self.auxes):
-            if isinstance(aux, Mapping):  # Indicates an insular clone  (see builder.buildAux)
-                clone = aux['clone']
-                original = aux['original']
-                schedule = aux['schedule']
-                human = aux['human']
-                count = aux['count']
-                inode = aux['inode']
-                if schedule != AUX:
-                    msg = ("ResolveError: Invalid insular clone schedule '{0}' "
-                           "for {1}.".format(schedule, original))
-                    raise excepting.ResolveError(msg,
-                                                 name=clone,
-                                                 value=self.name,
-                                                 human=human,
-                                                 count=count)
-                if clone != 'mine':
-                    msg = "Aux insular clone name must be 'mine' not '{0}'".format(clone)
-                    raise excepting.ResolveError(msg,
-                                                 name=clone,
-                                                 value=self.name,
-                                                 human=human,
-                                                 count=count)
-
-                clone = Framer.nameUid(prefix=original)
-                while (clone in Framer.Names): # ensure unique
-                    clone = Framer.nameUid(prefix=original)
-
-                console.terse("         Cloning original '{0}' as insular clone '{1}'\n"
-                                            "".format(original, clone))
-                original = resolveFramer(original,
-                                         who=self.name,
-                                         desc='original',
-                                         contexts=[MOOT],
-                                         human=human,
-                                         count=count)
-                clone = original.clone(name=clone, schedule=schedule)
-                self.framer.assignFrameRegistry()  # restore original.clone changes above
-                clone.original = False  # main frame will be fixed
-                clone.insular =  True #  local to this framer
-
-                # inode is new (aux verb clone via)  clone.inode is old (framer moot via)
-                if inode:  # new != "" so resultant is new
-                    clone.inode = inode  # replace old with new
-
-                self.auxes[i] = aux = clone
-                self.store.house.resolvables.append(clone)
-            else:  # not insular has given name
-                self.auxes[i] = aux = resolveFramer(aux,
-                                                who=self.name,
-                                                desc='aux',
-                                                contexts=[AUX])
-
-            if not aux.original: # clones get fixed main never released
-                if aux.main: # raise exception if aux.main is not None
-                    msg = "Aux already assigned to main '{0}'".format(aux.main.name)
-                    raise excepting.ResolveError(msg,
-                                                 name=aux.name,
-                                                 value=self.name)
-                aux.main = self
-
     def expose(self):
         """Prints out instance variables.
 
@@ -1399,6 +1508,49 @@ class Frame(registering.StoriedRegistrar):
 
 
 #utility functions
+def resolveAuxOfFramer(aux,
+                       framer,
+                       who='',
+                       desc='aux',
+                       contexts=None,
+                       human='',
+                       count=None):
+    """ Returns resolved aux framer instance from framer
+        aux is either name of aux framer to resolve or pre-resolved instance
+        framer is framer instance that contains aux.
+        who is optional name of object owning the aux framer link
+            such as main framer or main frame or actor
+        desc is string description of framer link such as 'aux' or 'clone'
+        contexts is list of allowed schedule contexts, None or empty means any.
+        human is human readable version of associated declaration
+        count is line number of associated declaration
+
+        Framer.Names registry must already be setup
+    """
+    if not isinstance(aux, Framer): # not instance so aux is a name or tag
+        if aux not in framer.auxes:
+            msg = ("ResolveError: Bad {0} link name '{1}'".format(desc, aux))
+            raise excepting.ResolveError(msg,
+                                         framer.name,
+                                         who,
+                                         human,
+                                         count)
+        aux = framer.auxes[aux]  # replace name with instance
+
+        if contexts and aux.schedule not in contexts:
+            raise excepting.ResolveError("ResolveError: Bad {0} link not scheduled"
+                                         " as one of {1}".format(desc,
+                                            [ScheduleNames.get(context, context)
+                                                 for context in contexts]),
+                                         aux.name,
+                                         who,
+                                         human,
+                                         count)
+        console.concise("         Resolved {0} Framer '{1}' with tag '{2}' in {3}\n"
+                      "".format(desc, aux.name, aux.tag, who))
+    return aux
+
+
 def resolveFramer(framer, who='', desc='framer', contexts=None,
                   human='', count=None):
     """ Returns resolved framer instance from framer
